<?php

declare(strict_types=1);

namespace UnifyPayment\Cores\Traits;

use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use InvalidArgumentException;
use Psr\Http\Message\ResponseInterface;
use UnifyPayment\Cores\BasicClient;

trait HttpRequest
{
    /**
     * Http client.
     *
     * @var Client|null
     */
    protected ?Client $httpClient = null;

    /**
     * Mock handler.
     *
     * @var MockHandler|null
     */
    protected ?MockHandler $mockHandler = null;

    /**
     * Http header
     *
     * @var array
     */
    private array $httpHeader = [];

    /**
     * Http client options.
     *
     * @var array
     */
    protected array $httpOptions = [
        'base_uri' => '',
        'timeout' => 0,
        'connect_timeout' => 0
    ];

    /**
     * Send a GET request.
     *
     * @return array|string
     */
    public function get(string $endpoint, array $query = [], array $headers = [])
    {
        return $this->request('GET', $endpoint, [
            'headers' => $headers,
            'query' => $query,
        ]);
    }

    /**
     * Send a POST request.
     *
     * @param string|array $data
     *
     * @return array|string
     */
    public function post(string $endpoint, $data, array $options = [])
    {
        if (!is_array($data)) {
            $options['body'] = $data;
        } else {
            $options['form_params'] = $data;
        }

        return $this->request('POST', $endpoint, $options);
    }

    /**
     * Send request.
     *
     * @return array|string
     */
    public function request(string $method, string $endpoint, array $options = [])
    {
        return $this->unwrapResponse($this->getHttpClient()->{$method}($endpoint, $options));
    }

    /**
     * Convert response.
     *
     * @return array|string
     */
    public function unwrapResponse(ResponseInterface $response)
    {
        $contentType = $response->getHeaderLine('Content-Type');
        $contents = $response->getBody()->getContents();

        if (false !== stripos($contentType, 'json') || stripos($contentType, 'javascript')) {
            return json_decode($contents, true);
        }

        if (false !== stripos($contentType, 'xml')) {
            return json_decode(
                json_encode(
                    simplexml_load_string($contents, 'SimpleXMLElement', LIBXML_NOCDATA),
                    JSON_UNESCAPED_UNICODE
                ),
                true
            );
        }

        return $contents;
    }

    /**
     * Set http client.
     *
     * @return $this
     */
    public function setHttpClient(Client $client): self
    {
        $this->httpClient = $client;

        return $this;
    }

    /**
     * Get http client.
     *
     * @return Client
     */
    public function getHttpClient(): Client
    {
        if (is_null($this->httpClient)) {
            $this->httpClient = $this->getDefaultHttpClient();
        }

        return $this->httpClient;
    }

    /**
     * Get default http client.
     *
     * @return Client
     */
    public function getDefaultHttpClient(): Client
    {
        return new Client($this->getOptions());
    }

    /**
     * set mock Handler
     *
     * @param Response[] $mock
     *
     * @return HttpRequest|BasicClient
     */
    public function setMockHandler(array $mock): self
    {
        array_walk($mock, static function ($value) {
            if (!is_subclass_of($value, ResponseInterface::class)) {
                throw new InvalidArgumentException(
                    $value . ' must be an instance of ' . ResponseInterface::class
                );
            }
        });

        $this->mockHandler = new MockHandler($mock);
        $handlerStack = HandlerStack::create($this->mockHandler);
        $this->setHttpOptions(['handler' => $handlerStack]);

        return $this;
    }

    /**
     * Get mock handler
     *
     * @return MockHandler|null
     */
    public function getMockHandler(): ?MockHandler
    {
        return $this->mockHandler;
    }

    /**
     * Get default options.
     *
     * @return array
     */
    public function getOptions(): array
    {
        return $this->getHttpOptions();
    }

    /**
     * Set http header.
     *
     * @return $this
     */
    public function setHttpHeader(array $httpHeader): self
    {
        $this->httpHeader = array_merge($httpHeader, $this->httpHeader);

        return $this;
    }

    /**
     * Get http header.
     *
     * @return array
     */
    public function getHttpHeader(): array
    {
        return $this->httpHeader;
    }

    /**
     * Set http options.
     *
     * @param array $httpOptions
     *
     * @return void
     */
    public function setHttpOptions(array $httpOptions): void
    {
        $this->httpOptions = array_merge($this->httpOptions, $httpOptions);
    }

    /**
     * Get http options.
     *
     * @return array
     */
    public function getHttpOptions(): array
    {
        return $this->httpOptions;
    }
}